import pyfie
import matplotlib.pyplot as plt

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 処理対象画像の読み込み
image = pyfie.imread("houghline_input.png")

# ソーベルフィルタを利用したエッジ検出
edges = pyfie.F_DEDGE.PTR()  # 取得したエッジ
nedges = pyfie.INT()  # 取得したエッジ数
pyfie.fnFIE_edge_sobel_subpix(
    image, None, pyfie.F_EDGE_SOBEL_PARAMS(mag_threshold=40, nms_length=1),
    pyfie.F_EDGE_FEAT_DIRECT, pyfie.F_BORDER_NONE, (0.0, 0.0),
    edges, nedges
)

# 画像オブジェクトのプロット
image.imshow()
# エッジ点をプロット
edges.plot(nedges, c="lime", s=1)
# プロットした画像の表示
pyfie.plot_show()

# 直線検出ハフライブラリのオープン
hhough = pyfie.fnFIE_hough_lines_open(
    0, 359, 0, 0, image.width, image.height, None)
assert hhough is not None

# 直線ハフ投票空間の初期化
pyfie.fnFIE_hough_lines_init_space(hhough)

# エッジの投票用パラメータ設定
vot_width = 10  # 投票θ片幅

# エッジの投票
nvoted = pyfie.INT()  # 投票されたエッジ数
pyfie.fnFIE_hough_lines_voting_d(hhough, edges, nedges, vot_width, nvoted)

# 直線検出のパラメータを設定
rgn_r = 10  # ρ方向極大値決定サイズ
rgn_q = 10  # θ方向極大値決定サイズ
max_lines = 10  # 検出要求数

# データ受け取り用変数を用意
detedted_lines = pyfie.F_LH_LINE.ARRAY(max_lines)  # 検出結果直線
nlines = pyfie.INT()  # 検出直線の数

# 直線の検出
pyfie.fnFIE_hough_lines_detection(
    hhough, max_lines, rgn_r, rgn_q, detedted_lines, nlines)

# 取得した直線を表示
print("num lines:", nlines)
for i in range(nlines):
    line = detedted_lines[i]
    print("#{}: a = {:.2f}, b = {:.2f}, c = {:.2f}, score = {}".format(
        i, line.a, line.b, line.c, line.score
    ))

# 画像オブジェクトのプロット
image.imshow()
# 取得した直線をプロット
detedted_lines.plot(nedges, num=nlines, color="red")
# 結果をプロットした画像の保存
plt.savefig("hough_line_result.png")
# 結果をプロットした画像の表示
pyfie.plot_show()

# エッジを解放
edges.fnOAL_free()
